package com.netapp.platform.twiddle;

import gnu.getopt.Getopt;
import gnu.getopt.LongOpt;

import java.text.MessageFormat;

public class Twiddle {

	private static String SERVICE_PORT = "33333";
	private static String SERVICE_URL = "service:jmx:rmi:///jndi/rmi://localhost:{0}/jmxrmi";
	private static String PROGRAM_NAME = System.getProperty("program.name", "twiddle");
	private static String commandName;
	private static String[] commandArgs;

	public static void main(String[] args) {

		try {
			processArguments(args);

			if (commandName == null) {
				displayHelp();
				System.exit(1);
			}

            if (SERVICE_URL.contains("{0}")) {
                SERVICE_URL = MessageFormat.format(SERVICE_URL, SERVICE_PORT);
            }
			JmxExecutor executor = new JmxExecutor(SERVICE_URL);

			String result = null;

			if (commandName.equalsIgnoreCase("invoke")) {
				result = executor.invoke(commandArgs);

			} else if (commandName.equalsIgnoreCase("get")) {
				result = executor.get(commandArgs);

			} else {
				displayHelp();
				System.exit(1);
			}

			if (result !=null) {
				System.out.println(result);
			}
			executor.destroy();

		} catch (Exception e) {
			e.printStackTrace(System.out);
                        System.exit(2);
		}
	}

	private static void displayHelp() {
		System.out.println("A JMX client to 'twiddle' with a local wildfly server.");
		System.out.println();
		System.out.println("usage: " + PROGRAM_NAME + " [options] <command> <command_arguments>");
		System.out.println();
		System.out.println("options:");
		System.out.println("    -h, --help                Show this help message");
		System.out.println("    -p, --port=<number>       The JMX RMI server port number, defaults to 33333");
		System.out.println("    -s, --server=<url>        The JNDI URL of the local wildfly server");
		System.out.println();
		System.out.println("command:");
		System.out.println("    invoke <beanName> <operationName> [args] - Invoke the operationName with optional args on the specificied beanName. [Overloaded operation is not supported]");
		System.out.println("    get <beanName> <attributeName[,attributeName...]> - Get value of the attributeName(s) on the specified beanName");
		System.out.println();
		System.out.println("note:");
		System.out.println("    -s will override -p if both are specified on the command line.");
	}

	private static void processArguments(final String[] args) throws Exception {
		String shortOptions = "-p:s:";

		LongOpt[] longOptions = {
			new LongOpt("help", LongOpt.NO_ARGUMENT, null, 'h'),
			new LongOpt("port", LongOpt.OPTIONAL_ARGUMENT, null, 'p'),
			new LongOpt("server", LongOpt.OPTIONAL_ARGUMENT, null, 's'),
		};

		Getopt getopt = new Getopt(PROGRAM_NAME, args, shortOptions, longOptions);
		int code;
        boolean mutex = false;

		PROCESS_ARGUMENTS:

		while ((code = getopt.getopt()) != -1) {
			switch (code) {
				case ':':
				case '?':
					System.exit(1);
					break;

					// non-option arguments
				case 1:
				{
					commandName = getopt.getOptarg();

					// pass the remaining arguments (if any) to the command for processing
					int i = getopt.getOptind();

					if (args.length > i) {
						commandArgs = new String[args.length - i];
						System.arraycopy(args, i, commandArgs, 0, args.length - i);
					} else {
						commandArgs = new String[0];
					}

					if (commandName.equalsIgnoreCase("get") && commandArgs.length < 2) {
						displayHelp();
						System.exit(1);
					}

					if (commandName.equalsIgnoreCase("invoke") && commandArgs.length < 2) {
						displayHelp();
						System.exit(1);
					}
					break PROCESS_ARGUMENTS;
				}

				case 'p':
					SERVICE_PORT = getopt.getOptarg();
					break;

				case 's':
					SERVICE_URL = getopt.getOptarg();
					break;

				case 'h':
					displayHelp();
					System.exit(0);
	           }
	      }
	}
}
